from edgegap_scheduling import SchedulingSingleton, Task, errors
from fastapi import (
    FastAPI,
    HTTPException
)
from fastapi.routing import APIRouter


class SchedulingAPI:
    tags = ["Scheduling"]

    def __init__(self):
        self.__instance = SchedulingSingleton.scheduler()

    def init_scheduling_api(self, app: FastAPI):
        router = APIRouter(prefix="/scheduling/tasks", tags=self.tags)

        @router.get("/", description="Get all Tasks")
        async def get_tasks() -> list[Task]:
            return await self.__instance.list()

        @router.get("/{identifier}", description="Get One Task")
        async def get_task(identifier: str) -> Task:
            try:
                return await self.__instance.get(identifier)
            except ValueError as e:
                raise HTTPException(status_code=404, detail=str(e))

        @router.post("/{identifier}", description="Start a stopped Task")
        async def start_task(identifier: str) -> Task:
            try:
                return await self.__instance.start(identifier)
            except ValueError as e:
                raise HTTPException(status_code=404, detail=str(e))

        @router.patch("/{identifier}", description="Update the Interval of a Task")
        async def update_task(identifier: str, every: str | None = None, delay: str | None = None) -> Task:
            try:
                return await self.__instance.update(identifier, every, delay)
            except ValueError as e:
                raise HTTPException(status_code=404, detail=str(e))

        @router.delete("/{identifier}", description="Stop a running Task")
        async def stop_task(identifier: str) -> Task:
            try:
                return await self.__instance.stop(identifier)
            except ValueError as e:
                raise HTTPException(status_code=404, detail=str(e))

        @router.post("/run/{identifier}")
        async def run_rask(identifier: str) -> Task:
            try:
                return await self.__instance.run(identifier)
            except ValueError as e:
                raise HTTPException(status_code=404, detail=str(e))
            except errors.ManualRunNotAllowedError as e:
                raise HTTPException(status_code=403, detail=str(e))

        app.include_router(router)



from edgegap_settings import SettingsFactory
from pydantic import BaseModel

from ._environment import EnvironmentConfiguration

env_config: EnvironmentConfiguration = SettingsFactory.from_settings(EnvironmentConfiguration)


class LoggingConfiguration(BaseModel):
    """Logging configuration to be set for the server"""

    # Logging config
    version: int = 1
    disable_existing_loggers: bool = False
    formatters: dict = {
        "default": {
            "()": "edgegap_logging.DefaultFormatter",
            "fmt": env_config.log_format,
        },
        "access": {
            "()": "edgegap_logging.AccessFormatter",
            "fmt": env_config.log_access_format,
        },
    }
    handlers: dict = {
        "default": {
            "formatter": "default",
            "class": "logging.StreamHandler",
            "stream": "ext://sys.stdout",
        },
        "access": {
            "formatter": "access",
            "class": "logging.StreamHandler",
            "stream": "ext://sys.stdout",
        }
    }
    loggers: dict = {
        "root": {
            "handlers": ["default"],
            "level": env_config.log_level
        },
        "uvicorn": {
            "handlers": ["default"],
            "level": env_config.log_level,
            "propagate": False
        },
        "uvicorn.error": {
            "level": env_config.log_level
        },
        "uvicorn.access": {
            "handlers": ["access"],
            "level": env_config.log_level,
            "propagate": False
        },
        "gunicorn.error": {
            "level": env_config.log_level,
            "handlers": ["default"],
            "propagate": False,
        },
        "gunicorn.access": {
            "level": env_config.log_level,
            "handlers": ["access"],
            "propagate": False,
        },
        "elasticapm": {
            "level": "INFO",
            "handlers": ["default"],
            "propagate": False,
        },
        "urllib3": {
            "level": "INFO",
            "handlers": ["default"],
            "propagate": False,
        }

    }
    root: dict = {
        'level': env_config.log_level,
        'handlers': ['default'],
    }



from pydantic import BaseModel, Field


class CheckInstanceModel(BaseModel):
    ok: bool = Field(default=True, description="If the Check is OK")
    message: str = Field(default="ok", description="The message of the Check")


class CheckModel(CheckInstanceModel):
    name: str = Field(..., description="The name of the Check")
